@@ -47,13 +47,20 @@ class Info(ControllerResource):
                         'scan': self.controller.get_results()})
 
 
+scan_post_parser = RequestParser()
+scan_post_parser.add_argument('positions', type=list, required=True,
+                              help='list of (x, y) positions, example [(0.0, 0.0), (1.0, 0.0), ...]',
+                              location=['json'])
+
+
 class ScanStart(ControllerResource):
 
     def post(self):
+        args = scan_post_parser.parse_args()
         if self.controller.is_scan_command_running():
             abort(403, message='scan command already running')
         try:
-            self.controller.start_scan()
+            self.controller.start_scan(positions=args.positions)
         except AlreadyScanningException:
             abort(403, message='scan command already running')
 
@@ -74,6 +81,7 @@ except Exception as e:
     controller.shutdown_all_systems()
     raise
 
+
 while True:
     status = controller.get_connection_status()
     logger.info('Phootonics controller status: {}'.format(status))
