@@ -13,14 +13,12 @@ import time
 from time import sleep
 
 from phootonics_controller.base_controllers.config import VCM_PORT, CAVITY1_INDEX, CAVITY2_INDEX, CAVITY3_INDEX, \
-    CAVITY1_URL, CAVITY2_URL, CAVITY3_URL, ADC_CHANNEL, REF, configuration_dictionary
+    CAVITY1_URL, CAVITY2_URL, CAVITY3_URL, ADC_CHANNEL, REF, configuration_dictionary, CAVITY4_URL, CAVITY4_INDEX, \
+    EXT_VCM_PORT, get_scanning_wavelengths
 from phootonics_controller.base_controllers.stage_mock import MockStageController
 from phootonics_controller.base_controllers.xc_controller import ECController, MockECController
 from phootonics_controller.base_controllers.stage_vcm import VCMirrorController
 
-
-
-
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.DEBUG)
 
@@ -31,6 +29,7 @@ def ignore_exception(f):
             return f(*args, **kargs)
         except Exception as e:
             logger.exception(e)
+
     return g
 
 
@@ -68,11 +67,10 @@ class MockADCDevice:
 
 class ScanCommand:
 
-    def __init__(self, controller, wavelegths, x_positions, y_positions):
+    def __init__(self, controller, wavelegths, positions):
         self._controller = controller
         self.wavelengths = wavelegths
-        self.listXPos = x_positions
-        self.listYPos = y_positions
+        self.positions = positions
         self.results = {}
         self.stopped = threading.Event()
         self.currentX = None
@@ -81,10 +79,45 @@ class ScanCommand:
         self.currentWavelength = None
         self.detectorVoltage = None
         self.error = None
+        self._testWithMocks = True
 
     def execute(self):
         try:
-            if self._controller.testMode:
+            if not self._controller.testMode or self._testWithMocks:
+                self._controller.move_to_cavity_1()
+                self._controller.get_active_cavity().activate_s2()
+                for wavelength in self.wavelengths:
+                    if self.stopped.is_set():
+                        return
+                    logger.info('going to wavelength {}'.format(wavelength))
+                    cavity_index = self._controller.get_cavity_from_wavelength(wavelength)
+                    if cavity_index is None:
+                        logger.info('cannot reach wavelength {}: skip to next wavelength'.format(wavelength))
+                        continue
+                    logger.info('selecting cavity {}'.format(cavity_index))
+                    self._controller.select_cavity(cavity_index)
+                    cavity = self._controller.get_cavity_from_index(cavity_index)
+                    cavity.move_to_wavelength(wavelength)
+                    for posX, posY in self.positions:
+                        if self.stopped.is_set():
+                            return
+                        self._controller.extVcm.move_to_angle_y(posY)
+                        self._controller.extVcm.move_to_angle(posX)
+                        currentX = self._controller.extVcm.get_angle()
+                        currentY = self._controller.extVcm.get_angle_y()
+                        activeCavity = self._controller.get_active_cavity_index()
+
+                        detectorVoltage = self._controller.adc.read()
+                        self.results['active_cavity'].append(activeCavity)
+                        self.results['x_set_point'].append(posX)
+                        self.results['x'].append(currentX)
+                        self.results['y_set_point'].append(posY)
+                        self.results['y'].append(currentY)
+                        self.results['wavelength_set_point'].append(wavelength)
+                        self.results['wavelength'].append(cavity.get_wavelength())
+                        self.results['detector_voltage'].append(detectorVoltage)
+
+            else:
                 if not self.stopped.wait(2.0):
                     self.results = {'active_cavity': [1, 1, 2, 2, 3, 3, 4, 4],
                                     'x_set_point': [0, 1, 0, 1, 0, 1, 0, 1],
@@ -94,6 +127,7 @@ class ScanCommand:
                                     'wavelength_set_point': [1300, 1301, 1302, 1303, 1304, 1305, 1306, 1307],
                                     'wavelength': [1300.2, 1301.1, 1302.4, 1303.3, 1304.2, 1305.6, 1306.2, 1307.1],
                                     'detector_voltage': [0.2, 0.5, 0.3, 0.11, 0.2, 0.5, 0.3, 0.11]}
+
         except Exception as e:
             self.error = str(e)
 
@@ -125,18 +159,20 @@ class MainController:
         self.testMode = test_mode
         if not self.testMode:
             self.vcm = VCMirrorController(VCM_PORT)
+            self.extVcm = VCMirrorController(EXT_VCM_PORT)
             self.adc = ADCDevice()
-            self.ec1 = ECController(ec_url=CAVITY1_URL, monitor_interval=0.3, index =CAVITY1_INDEX)
-            self.ec2 = ECController(ec_url=CAVITY2_URL, monitor_interval=0.3, index =CAVITY2_INDEX)
-            self.ec3 = ECController(ec_url=CAVITY3_URL, monitor_interval=0.3, index =CAVITY3_INDEX)
-            # self.ec4 = ECController(ec_url=CAVITY4_URL, monitor_interval=1, index =CAVITY4_INDEX)
+            self.ec1 = ECController(ec_url=CAVITY1_URL, monitor_interval=0.3, index=CAVITY1_INDEX)
+            self.ec2 = ECController(ec_url=CAVITY2_URL, monitor_interval=0.3, index=CAVITY2_INDEX)
+            self.ec3 = ECController(ec_url=CAVITY3_URL, monitor_interval=0.3, index=CAVITY3_INDEX)
+            self.ec4 = ECController(ec_url=CAVITY4_URL, monitor_interval=0.3, index=CAVITY4_INDEX)
         else:
             self.vcm = MockStageController()
+            self.extVcm = MockStageController()
             self.adc = MockADCDevice()
-            self.ec1 = MockECController(monitor_interval=0.3, index =CAVITY1_INDEX)
-            self.ec2 = MockECController(monitor_interval=0.3, index =CAVITY2_INDEX)
-            self.ec3 = MockECController(monitor_interval=0.3, index =CAVITY3_INDEX)
-            # self.ec4 = ECController(ec_url=CAVITY4_URL, monitor_interval=1, index =CAVITY4_INDEX)
+            self.ec1 = MockECController(monitor_interval=0.3, index=CAVITY1_INDEX)
+            self.ec2 = MockECController(monitor_interval=0.3, index=CAVITY2_INDEX)
+            self.ec3 = MockECController(monitor_interval=0.3, index=CAVITY3_INDEX)
+            self.ec4 = MockECController(monitor_interval=0.3, index=CAVITY4_INDEX)
         self._active_cavity = None
         self._monitoring_data = {}
         self._recorded_data = {}
@@ -155,29 +191,29 @@ class MainController:
         self._ec2_ready_status = None
         self._ec3_connection_status = None
         self._ec3_ready_status = None
-        # self._ec4_connection_status = None
-        # self._ec4_ready_status = None
+        self._ec4_connection_status = None
+        self._ec4_ready_status = None
         self._vcm_status = None
+        self._extVcm_status = None
 
     def get_cavity_from_wavelength(self, wavelength):
         for key, item in configuration_dictionary.items():
             if item['wavelength'][0] <= wavelength <= item['wavelength'][1]:
                 return key
-        raise Exception('No cavity for wavelength {}'.format(wavelength))
+        return None
 
     def get_results(self):
         if self._scanCommand:
             return self._scanCommand.to_dict()
 
-    def start_scan(self):
+    def start_scan(self, wavelengths=None, positions=None):
         if self._scanCommandThread:
             if self._scanCommandThread.is_alive():
                 raise AlreadyScanningException
             self._scanCommandThread = None
         self._scanCommand = ScanCommand(controller=self,
-                                        wavelegths=None,
-                                        x_positions=None,
-                                        y_positions=None)
+                                        wavelegths=wavelengths or get_scanning_wavelengths(),
+                                        positions=positions)
         self._scanCommandThread = threading.Thread(target=self._scanCommand.execute)
         self._scanCommandThread.start()
 
@@ -199,8 +235,10 @@ class MainController:
             logging.info('Cavity 2 initialized')
             self.ec3.start()
             logging.info('Cavity 3 initialized')
-            # self.ec4.start()
+            self.ec4.start()
+            logging.info('Cavity 4 initialized')
             self.vcm.start()
+            self.extVcm.start()
             logging.info('cavities initialized')
             self.adc.initialise()
             logging.info('system initialized')
@@ -212,8 +250,9 @@ class MainController:
     def get_connection_status(self):
         if self._ec1_connection_status == 'RESPONDING' and self._ec2_connection_status == 'RESPONDING' and \
                 self._ec3_connection_status == 'RESPONDING' and \
-                self._vcm_status not in [None, 'DISCONNECTED', 'CONNECTION_ERROR']:
-                # self._ec4_connection_status == 'RESPONDING' and \
+                self._vcm_status not in [None, 'DISCONNECTED', 'CONNECTION_ERROR'] and \
+                self._extVcm_status not in [None, 'DISCONNECTED', 'CONNECTION_ERROR'] and \
+                self._ec4_connection_status == 'RESPONDING':
             return MainController.RESPONDING
         else:
             return MainController.NOT_RESPONDING
@@ -221,8 +260,9 @@ class MainController:
     def is_ready_to_action(self):
         if self._ec1_ready_status == 'READY' and self._ec2_ready_status == 'READY' and \
                 self._ec3_ready_status == 'READY' and \
-                self._vcm_status == 'READY':
-                # and self._ec4_ready_status == 'READY'
+                self._vcm_status == 'READY' and \
+                self._extVcm_status == 'READY' and \
+                self._ec4_ready_status == 'READY':
             return MainController.READY
         else:
             return MainController.NOT_READY
@@ -234,12 +274,14 @@ class MainController:
                 self.ec1.shutdown()
                 self.ec2.shutdown()
                 self.ec3.shutdown()
+                self.ec4.shutdown()
             else:
                 self.ec1.stop_pulsing()
                 self.ec2.stop_pulsing()
                 self.ec3.stop_pulsing()
-            # self.ec4.shutdown()
+                self.ec4.stop_pulsing()
             self.vcm.stop()
+            self.extVcm.stop()
         except Exception as e:
             logging.error('error on shutdown {}'.format(e))
 
@@ -256,8 +298,8 @@ class MainController:
             return self.ec2
         elif cavity_index == CAVITY3_INDEX:
             return self.ec3
-        # elif cavity_index == CAVITY4_INDEX:
-        #     return self.ec4
+        elif cavity_index == CAVITY4_INDEX:
+            return self.ec4
 
     def stop_monitoring(self):
         self._stop_monitoring_event.set()
@@ -276,8 +318,8 @@ class MainController:
         ec1_monitoring_trace = None
         ec2_monitoring_trace = None
         ec3_monitoring_trace = None
+        ec4_monitoring_trace = None
         t = None
-        # ec4_monitoring_trace = None
         while self._stop_monitoring_event.wait(self._monitoring_loop_delay) is False:
             if not self._stop_monitoring_event.is_set():
                 device_time = time.time()
@@ -291,13 +333,14 @@ class MainController:
                 self._ec2_ready_status = ignore_exception(self.ec2.xc_ready_to_action)()
                 self._ec3_connection_status = self.ec2.status
                 self._ec3_ready_status = ignore_exception(self.ec2.xc_ready_to_action)()
-                # self._ec4_connection_status = self.ec2.status
-                # self._ec4_ready_status = ignore_exception(self.ec2.xc_ready_to_action)()
+                self._ec4_connection_status = self.ec2.status
+                self._ec4_ready_status = ignore_exception(self.ec2.xc_ready_to_action)()
                 self._vcm_status = self.vcm.status
+                self._extVcm_status = self.extVcm.status
                 ec1_monitoring_trace = ignore_exception(self.ec1.system_info)()['response']['data']
                 ec2_monitoring_trace = ignore_exception(self.ec2.system_info)()['response']['data']
                 ec3_monitoring_trace = ignore_exception(self.ec3.system_info)()['response']['data']
-                # ec4_monitoring_trace = ignore_exception(self.ec4.system_info)()['response']['data']
+                ec4_monitoring_trace = ignore_exception(self.ec4.system_info)()['response']['data']
             self._monitoring_data = {'t': t,
                                      'active_cavity': self._system_active_cavity,
                                      'system_ready_status': self._system_ready_status,
@@ -305,16 +348,17 @@ class MainController:
                                      'ec1_connection': self._ec1_connection_status,
                                      'ec2_connection': self._ec2_connection_status,
                                      'ec3_connection': self._ec3_connection_status,
-                                     # 'ec4_connection': self._ec4_connection_status,
+                                     'ec4_connection': self._ec4_connection_status,
                                      'ec1_ready': self._ec1_ready_status,
                                      'ec2_ready': self._ec2_ready_status,
                                      'ec3_ready': self._ec3_ready_status,
-                                     # 'ec4_ready': self._ec4_ready_status,
+                                     'ec4_ready': self._ec4_ready_status,
                                      'ec1_trace': ec1_monitoring_trace,
                                      'ec2_trace': ec2_monitoring_trace,
                                      'ec3_trace': ec3_monitoring_trace,
-                                     # 'ec4_trace': ec4_monitoring_trace,
+                                     'ec4_trace': ec4_monitoring_trace,
                                      'vcm_status': self._vcm_status,
+                                     'extVcm_status': self._extVcm_status
                                      }
 
     def select_cavity(self, cavity_index):
@@ -322,7 +366,7 @@ class MainController:
             if self._active_cavity != cavity_index:
                 logger.info('switching to cavity {}'.format(cavity_index))
                 self._switch_and_activate_cavity(cavity_index, configuration_dictionary[cavity_index]['x_position'],
-                                           configuration_dictionary[cavity_index]['y_position'])
+                                                 configuration_dictionary[cavity_index]['y_position'])
         else:
             raise Exception('Controller not responding')
 
@@ -343,7 +387,8 @@ class MainController:
             self._active_cavity = cavity_index
         except Exception as e:
             logging.error('Could not move to cavity {} at position {} {}, error {}'.format(cavity_index,
-                                                                                        cavity_position_x, cavity_position_y, e))
+                                                                                           cavity_position_x,
+                                                                                           cavity_position_y, e))
 
     def move_to_cavity_1(self):
         try:
@@ -355,5 +400,3 @@ class MainController:
 
     def get_adc_voltage(self):
         return self.adc.read()
-
-
